import Flutter
import UIKit
import Passbase

private enum FlutterBridgeMethod: String {
    case initialize
    case startVerification
    case setPrefillUserEmail
}

public class SwiftPassbaseFlutterPlugin: NSObject, FlutterPlugin {
    let channel: FlutterMethodChannel
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "passbase_flutter",
                                           binaryMessenger: registrar.messenger())
        let instance = SwiftPassbaseFlutterPlugin(channel: channel)
        registrar.addMethodCallDelegate(instance,
                                        channel: channel)
        registrar.register(PassbaseButtonFactory(channel: channel), withId: "PassbaseButton")
    }
    
    // MARK: - Call handling
    public func handle(_ call: FlutterMethodCall,
                       result: @escaping FlutterResult) {
        guard let method = FlutterBridgeMethod(rawValue: call.method) else {
            result("Method is not implemented")
            return
        }
        
        switch method {
        case .initialize:
            PassbaseSDK.source = 3
            PassbaseSDK.initialize(publishableApiKey: call.arguments as? String ?? "")
            result(nil)
        case .startVerification:
            guard let viewController = UIApplication.shared.keyWindow?.rootViewController else {
                result("Can't get root view controller")
                return
            }
            do {
                PassbaseSDK.delegate = self
                try PassbaseSDK.startVerification(from: viewController)
            } catch {
                result(error.localizedDescription)
            }
        case .setPrefillUserEmail:
            PassbaseSDK.prefillUserEmail = call.arguments as? String ?? ""
            return
        }
    }
    
    private func getColor(from arguments: [Int]) -> UIColor? {
        guard arguments.count >= 4 else {
            return nil
        }
        
        let r = arguments[0]
        let g = arguments[1]
        let b = arguments[2]
        let a = arguments[3]
        
        return UIColor(red: CGFloat(r) / 255,
                       green: CGFloat(g) / 255,
                       blue: CGFloat(b) / 255,
                       alpha: CGFloat(a) / 255)
    }
}

extension SwiftPassbaseFlutterPlugin: PassbaseDelegate {
    public func onFinish(identityAccessKey: String) {
        channel.invokeMethod("verification_complete", arguments: identityAccessKey)
    }
    
    public func onError(errorCode: String) {
        channel.invokeMethod("verification_error", arguments: errorCode)
    }
    
    public func onStart() {
        channel.invokeMethod("verification_start", arguments: "Verification started")
    }
}


class PassbaseButtonFactory: NSObject, FlutterPlatformViewFactory {
    let channel: FlutterMethodChannel
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
        
        super.init()
    }
    
    func create(withFrame frame: CGRect,
                viewIdentifier viewId: Int64,
                arguments args: Any?) -> FlutterPlatformView {
        return PassbaseButtonFlutter(frame, viewId: viewId, args: args, channel: channel)
    }
}

class PassbaseButtonFlutter: NSObject, FlutterPlatformView {
    let frame: CGRect
    let viewId: Int64
    let button = PassbaseButton()
    let channel: FlutterMethodChannel
    
    init(_ frame: CGRect, viewId: Int64, args: Any?, channel: FlutterMethodChannel) {
        self.frame = frame
        self.viewId = viewId
        button.frame = frame
        self.channel = channel
        
        super.init()
        
        PassbaseSDK.delegate = self
    }
    
    func view() -> UIView {
        return button
    }
}

extension PassbaseButtonFlutter: PassbaseDelegate {
    public func onFinish(identityAccessKey: String) {
        channel.invokeMethod("verification_complete", arguments: identityAccessKey)
    }
    
    public func onError(errorCode: String) {
        channel.invokeMethod("verification_error", arguments: errorCode)
    }
    
    public func onStart() {
        channel.invokeMethod("verification_start", arguments: "Verification started")
    }
}
